/*
  The main purpose of the following code is to handle both an RGB sensor and
  an SD Card module in order to take samples of different fruits  and register
  them in a file for a later analysis. 
  WIRING :
  A4 - SDA
  A5 - SCL

  04 - CS
  11 - MOSI
  12 - MISO
  13 - SCK
*/
#include <Wire.h>
#include "Adafruit_TCS34725.h"  //Library for th RGB Sensor

#include <SPI.h>
#include <SD.h>                 //Library for the SD Card Module
//PIN ASSIGNMENT
const int chipSelect = 10;       //Control Pin for the SD Card Module
int trigger = 2;
//VARIABLES DEFINITION
String fileName = "rgbData";
String extension = ".csv";
String file = fileName + extension;
int samples = 0;
Adafruit_TCS34725 tcs = Adafruit_TCS34725(TCS34725_INTEGRATIONTIME_24MS, TCS34725_GAIN_16X);

void setup() {
  Serial.begin(9600); //Start Serial Communication
  pinMode(trigger, INPUT_PULLUP);
  Serial.println("Perceptron Test");
  loadRGB();   //RGB Sensor Loading sequence
  loadSD();  //SD Card Module Loading sequence
  addHeaders();
}

void loop() {
  String dataString = ""; //Variable para escribir los datos en el archivo
  float red, green, blue; //Variables con el valor de cada color
  if (!digitalRead(trigger)) {
    samples += 1;
    //Lectura de color del sensor
    tcs.setInterrupt(false);    //Enciende el LED
    delay(60);                  //Le lleva 50ms leer
    tcs.setInterrupt(true);    //Apaga el LED
    tcs.getRGB(&red, &green, &blue);
    Serial.print("S:\t");      Serial.print(samples);
    Serial.print("\tR:\t");    Serial.print(red);
    Serial.print("\tG:\t");    Serial.print(green);
    Serial.print("\tB:\t");    Serial.println(blue);

    //Concatenación de los valores como cadenas de texto
    dataString = String(samples) + "," + String(red) + "," + String(green) + ","  + String(blue);
    File dataFile = SD.open(String(file), FILE_WRITE);
    // if the file is available, write to it:
    if (dataFile) {
      dataFile.println(dataString);
      dataFile.close();
      // print to the serial port too:
      Serial.println(dataString);
    }
    // if the file isn't open, pop up an error:
    else {
      Serial.println("error opening File");
    }
  }
}
void loadRGB() {
  Serial.println("Looking for a sensor");
  delay(100);
  Serial.println("Looking for a sensor.");
  delay(100);
  Serial.println("Looking for a sensor..");
  delay(100);
  Serial.println("Looking for a sensor...");
  delay(100);
    if (tcs.begin()) {
    Serial.println("Sensor Detected");
  } else {
    Serial.println("No sensor Available :(");
    while (1);
  }
}
void loadSD() {
  Serial.println("Initializing SD Module");
  delay(100);
  Serial.println("Initializing SD Module.");
  delay(100);
  Serial.println("Initializing SD Module..");
  delay(100);
  Serial.println("Initializing SD Module...");
  delay(100);
   // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    Serial.println("SD Card failed, or not present");
    // don't do anything more:
    while (1);
  }
  Serial.println("SD Card initialized");
}
void addHeaders() {
  File dataFile = SD.open(String(file), FILE_WRITE);
  String encabezados = "Sample,R,G,B";
  // if the file is available, write to it:
  if (dataFile) {
    dataFile.println(encabezados);
    dataFile.close();
    // print to the serial port too:
    Serial.println("los encabezados: " + encabezados + " fueron agregados");
  }
  // if the file isn't open, pop up an error:
  else {
    Serial.println("error opening File");
  }
}
